import { ApiRootLinkRel, ApiRootResource, ApiRootService } from '@alfa-client/api-root-shared';
import {
  HasLinkPipe,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import {
  Mock,
  existsAsHtmlElement,
  getElementFromFixture,
  mock,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { Router } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { AuthenticationService } from 'authentication';
import { NavigationComponent } from 'libs/admin-settings/src/lib/navigation/navigation.component';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { UserProfileButtonContainerComponent } from '../common/user-profile-button-container/user-profile.button-container.component';
import { UnavailablePageComponent } from '../pages/unavailable/unavailable-page/unavailable-page.component';
import { AppComponent } from './app.component';

describe('AppComponent', () => {
  let component: AppComponent;
  let fixture: ComponentFixture<AppComponent>;

  const adminHeaderSelector: string = getDataTestIdOf('admin-header');
  const buildVersionSelector: string = getDataTestIdOf('build-version');
  const userProfileButtonSelector: string = getDataTestIdOf('user-profile-button');
  const navigationSelector: string = getDataTestIdOf('navigation');
  const routerOutletSelector: string = getDataTestIdOf('router-outlet');

  const authenticationService: Mock<AuthenticationService> = {
    ...mock(AuthenticationService),
    login: jest.fn().mockResolvedValue(Promise.resolve()),
  };

  const router: Mock<Router> = mock(Router);
  const apiRootService: Mock<ApiRootService> = mock(ApiRootService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        AppComponent,
        MockComponent(NavigationComponent),
        MockComponent(UserProfileButtonContainerComponent),
        MockComponent(UnavailablePageComponent),
        HasLinkPipe,
      ],
      imports: [RouterTestingModule],
      providers: [
        {
          provide: AuthenticationService,
          useValue: authenticationService,
        },
        {
          provide: ApiRootService,
          useValue: apiRootService,
        },
        {
          provide: Router,
          useValue: router,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(AppComponent);
    component = fixture.componentInstance;
  });

  it(`should have as title 'admin'`, () => {
    const appTitle: string = fixture.componentInstance.title;

    expect(appTitle).toEqual('admin');
  });

  describe('ngOnInit', () => {
    it('should call authService login', () => {
      component.ngOnInit();

      expect(authenticationService.login).toHaveBeenCalled();
    });

    it('should call doAfterLoggedIn', async () => {
      component.doAfterLoggedIn = jest.fn();

      component.ngOnInit();
      await fixture.whenStable();

      expect(component.doAfterLoggedIn).toHaveBeenCalled();
    });

    describe('do after logged in', () => {
      it('should call apiRootService to getApiRoot', () => {
        component.doAfterLoggedIn();

        expect(apiRootService.getApiRoot).toHaveBeenCalled();
      });

      it('should navigate to default route', () => {
        component.doAfterLoggedIn();

        expect(router.navigate).toHaveBeenCalledWith(['/']);
      });
    });
  });

  it('show not show header if apiRoot is not loaded', () => {
    component.apiRootStateResource$ = of(createEmptyStateResource<ApiRootResource>());

    notExistsAsHtmlElement(fixture, adminHeaderSelector);
  });

  describe('user profile button', () => {
    beforeEach(() => {
      component.apiRootStateResource$ = of(createStateResource(createApiRootResource()));
    });

    it('should show if apiRoot exists', () => {
      fixture.detectChanges();

      existsAsHtmlElement(fixture, userProfileButtonSelector);
    });
  });

  describe('navigation', () => {
    beforeEach(() => {});
    it('should exist if configuration link exists', () => {
      component.apiRootStateResource$ = of(
        createStateResource(createApiRootResource([ApiRootLinkRel.CONFIGURATION])),
      );

      fixture.detectChanges();

      existsAsHtmlElement(fixture, navigationSelector);
    });

    it('should not exist if configuration resource not available', () => {
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, navigationSelector);
    });
  });

  describe('build version', () => {
    const apiResource: ApiRootResource = createApiRootResource();

    beforeEach(() => {
      component.apiRootStateResource$ = of(createStateResource(apiResource));
    });

    it('should show after apiRoot loaded', () => {
      fixture.detectChanges();

      const buildVersionElement = getElementFromFixture(fixture, buildVersionSelector);
      expect(buildVersionElement.textContent.trim()).toEqual(`Version: ${apiResource.version}`);
    });
  });

  describe('router outlet', () => {
    beforeEach(() => {});

    it('should exist if configuration resource available', () => {
      component.apiRootStateResource$ = of(
        createStateResource(createApiRootResource([ApiRootLinkRel.CONFIGURATION])),
      );

      fixture.detectChanges();

      existsAsHtmlElement(fixture, routerOutletSelector);
    });

    it('should not exist if configuration resource not available', () => {
      component.apiRootStateResource$ = of(createStateResource(createApiRootResource()));

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, routerOutletSelector);
    });
  });
});
