import { Injectable } from '@angular/core';
import KcAdminClient, { NetworkError } from '@keycloak/keycloak-admin-client';
import { TokenProvider } from '@keycloak/keycloak-admin-client/lib/client';
import GroupRepresentation from '@keycloak/keycloak-admin-client/lib/defs/groupRepresentation';
import { OAuthService } from 'angular-oauth2-oidc';
import { Observable, OperatorFunction, catchError, from, map, throwError } from 'rxjs';
import { Organisationseinheit, OrganisationseinheitError } from './user.model';
import { KEYCLOAK_CREATE_GROUPS_ERROR_STATUS } from './user.util';

@Injectable({
  providedIn: 'root',
})
export class UserRepository {
  constructor(
    private oAuthService: OAuthService,
    private kcAdminClient: KcAdminClient,
  ) {
    this.registerAccessTokenProvider();
  }

  private registerAccessTokenProvider(): void {
    this.kcAdminClient.registerTokenProvider(this.getTokenProvider());
  }

  private getTokenProvider(): TokenProvider {
    return {
      getAccessToken: () => Promise.resolve(this.oAuthService.getAccessToken()),
    };
  }

  public deleteOrganisationseinheit(id: string): Observable<void> {
    return from(this.kcAdminClient.groups.del({ id }));
  }

  public findOrganisationseinheitItems(): Observable<Organisationseinheit[]> {
    return from(this.kcAdminClient.groups.find({ briefRepresentation: false })).pipe(
      map((reps: GroupRepresentation[]) =>
        reps.map((rep: GroupRepresentation) =>
          this.mapGroupRepresentationToOrganisationseinheit(rep),
        ),
      ),
    );
  }

  mapGroupRepresentationToOrganisationseinheit(group: GroupRepresentation): Organisationseinheit {
    return {
      id: group.id,
      name: group.name,
      organisationseinheitIds: group.attributes?.organisationseinheitId ?? [],
    };
  }

  public saveOrganisationseinheit(organisationseinheit: Organisationseinheit): Observable<void> {
    return from(
      this.kcAdminClient.groups.update(
        { id: organisationseinheit.id },
        {
          name: organisationseinheit.name,
          attributes: { organisationseinheitId: organisationseinheit.organisationseinheitIds },
        },
      ),
    ).pipe(this.rethrowMappedGroupsError());
  }

  public createOrganisationseinheit(
    name: string,
    organisationseinheitIds: string[],
  ): Observable<Organisationseinheit> {
    return from(
      this.kcAdminClient.groups.create({
        name,
        attributes: { organisationseinheitId: organisationseinheitIds },
      }),
    ).pipe(
      map(
        ({ id }): Organisationseinheit => ({
          id,
          name,
          organisationseinheitIds: organisationseinheitIds,
        }),
      ),
      this.rethrowMappedGroupsError(),
    );
  }

  rethrowMappedGroupsError<T>(): OperatorFunction<T, T> {
    return catchError((err) => throwError(() => this.mapCreateGroupsNetworkError(err)));
  }

  mapCreateGroupsNetworkError(error: NetworkError): OrganisationseinheitError {
    return {
      errorType: KEYCLOAK_CREATE_GROUPS_ERROR_STATUS[error.response.status],
      detail: error.responseData['errorMessage'] ?? '',
    };
  }
}
