import {
  OrganisationsEinheitListResource,
  OrganisationsEinheitResource,
  OrganisationsEinheitService,
} from '@alfa-client/collaboration-shared';
import {
  StateResource,
  ToEmbeddedResourcesPipe,
  createStateResource,
} from '@alfa-client/tech-shared';
import {
  EventData,
  Mock,
  dialogRefMock,
  getMockComponent,
  mock,
  triggerEvent,
} from '@alfa-client/test-utils';
import { DialogRef } from '@angular/cdk/dialog';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import faker from '@faker-js/faker';
import { ButtonComponent, CloseIconComponent } from '@ods/system';
import {
  createOrganisationsEinheitListResource,
  createOrganisationsEinheitResource,
  createOrganisationsEinheitResources,
} from 'libs/collaboration-shared/test/organisations-einheit';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { SearchOrganisationsEinheitContainerComponent } from './search-organisations-einheit-container.component';
import { SearchOrganisationsEinheitFormComponent } from './search-organisations-einheit-form/search-organisations-einheit-form.component';

describe('SearchOrganisationsEinheitContainerComponent', () => {
  let component: SearchOrganisationsEinheitContainerComponent;
  let fixture: ComponentFixture<SearchOrganisationsEinheitContainerComponent>;

  const searchOrganisationsEinheitComp: string = getDataTestIdOf('search-organisations-einheit');

  const service: Mock<OrganisationsEinheitService> = mock(OrganisationsEinheitService);

  const organisationsEinheitResource: OrganisationsEinheitResource =
    createOrganisationsEinheitResource();
  const organisationsEinheitResources: OrganisationsEinheitResource[] =
    createOrganisationsEinheitResources();
  const organisationsEinheitListResource: OrganisationsEinheitListResource =
    createOrganisationsEinheitListResource(organisationsEinheitResources);
  const organisationsEinheitStateListResource: StateResource<OrganisationsEinheitListResource> =
    createStateResource(organisationsEinheitListResource);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        SearchOrganisationsEinheitContainerComponent,
        ToEmbeddedResourcesPipe,
        MockComponent(SearchOrganisationsEinheitFormComponent),
        MockComponent(ButtonComponent),
        MockComponent(CloseIconComponent),
      ],
      providers: [
        {
          provide: OrganisationsEinheitService,
          useValue: service,
        },
        {
          provide: DialogRef,
          useValue: dialogRefMock,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(SearchOrganisationsEinheitContainerComponent);
    component = fixture.componentInstance;
    component.organisationsEinheitStateListResource$ = of(organisationsEinheitStateListResource);
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call service', () => {
      component.ngOnInit();

      expect(service.getSearchResultList).toHaveBeenCalled();
    });
  });

  describe('onKeyDownHandler', () => {
    it('should prevent default behavior for enter key', () => {
      const keyboardEvent: KeyboardEvent = {
        ...new KeyboardEvent('enter'),
        key: 'Enter',
        preventDefault: jest.fn(),
      };

      component.onKeyDownHandler(keyboardEvent);

      expect(keyboardEvent.preventDefault).toHaveBeenCalled();
    });
  });

  describe('search organisationsEinheit component', () => {
    beforeEach(() => {
      component.organisationsEinheitStateListResource$ = of(organisationsEinheitStateListResource);
      fixture.detectChanges();
    });

    it('should be called with organisationsEinheiten', () => {
      const comp: SearchOrganisationsEinheitFormComponent =
        getMockComponent<SearchOrganisationsEinheitFormComponent>(
          fixture,
          SearchOrganisationsEinheitFormComponent,
        );

      expect(comp.organisationsEinheiten).toEqual(organisationsEinheitResources);
    });

    it('should call search on openSearchDialog output', () => {
      component.search = jest.fn();
      const searchBy: string = faker.random.word();
      const eventData: EventData<SearchOrganisationsEinheitContainerComponent> = {
        fixture,
        elementSelector: searchOrganisationsEinheitComp,
        name: 'search',
        data: searchBy,
      };

      triggerEvent(eventData);

      expect(component.search).toHaveBeenCalledWith(searchBy);
    });

    it('should call selectSearchResult on selectSearchResult output', () => {
      component.selectSearchResult = jest.fn();

      const eventData: EventData<SearchOrganisationsEinheitContainerComponent> = {
        fixture,
        elementSelector: searchOrganisationsEinheitComp,
        name: 'selectSearchResult',
        data: organisationsEinheitResource,
      };

      triggerEvent(eventData);

      expect(component.selectSearchResult).toHaveBeenCalledWith(organisationsEinheitResource);
    });

    it('should call clearSearchResult', () => {
      component.clearSearchResult = jest.fn();

      const eventData: EventData<SearchOrganisationsEinheitContainerComponent> = {
        fixture,
        elementSelector: searchOrganisationsEinheitComp,
        name: 'clearSearchResult',
        data: organisationsEinheitResource,
      };

      triggerEvent(eventData);

      expect(component.clearSearchResult).toHaveBeenCalled();
    });
  });

  describe('search', () => {
    const searchBy: string = faker.random.word();

    it('should call service', () => {
      component.search(searchBy);

      expect(service.search).toHaveBeenCalledWith(searchBy);
    });
  });

  describe('select search result', () => {
    it('should set select result', () => {
      component.selectSearchResult(organisationsEinheitResource);

      expect(service.selectSearchResult).toHaveBeenCalledWith(organisationsEinheitResource);
    });

    it('should call service', () => {
      component.selectSearchResult(organisationsEinheitResource);

      expect(service.clearSearchResult).toHaveBeenCalled();
    });

    it('should close dialog', () => {
      component.selectSearchResult(organisationsEinheitResource);

      expect(dialogRefMock.close).toHaveBeenCalled();
    });
  });

  describe('clear search result', () => {
    it('should call service', () => {
      component.clearSearchResult();

      expect(service.clearSearchResult).toHaveBeenCalled();
    });
  });

  describe('close dialog', () => {
    it('should clear search result', () => {
      component.clearSearchResult = jest.fn();

      component.closeDialog();

      expect(component.clearSearchResult).toHaveBeenCalled();
    });

    it('should close dialog', () => {
      component.closeDialog();

      expect(dialogRefMock.close).toHaveBeenCalled();
    });
  });
});
