import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { fakeAsync, tick } from '@angular/core/testing';
import { Resource } from '@ngxp/rest';
import { Observable, of, throwError } from 'rxjs';
import { singleCold, singleHot } from '../../../test//marbles';
import { createProblemDetail } from '../../../test/error';
import { createDummyResource } from '../../../test/resource';
import { HttpError, ProblemDetail } from '../tech.model';
import { ApiResourceService } from './api-resource.service';
import { LinkRelationName, ResourceServiceConfig, SaveResourceData } from './resource.model';
import { ResourceRepository } from './resource.repository';
import { StateResource, createStateResource } from './resource.util';

describe('ApiResourceService', () => {
  let service: ApiResourceService<Resource, Resource>;
  let config: ResourceServiceConfig<Resource>;
  let repository: Mock<ResourceRepository>;

  const configResource: Resource = createDummyResource();
  const configStateResource: StateResource<Resource> = createStateResource(configResource);
  const configStateResource$: Observable<StateResource<Resource>> = of(configStateResource);

  const editLinkRel: string = 'dummyEditLinkRel';
  const getLinkRel: LinkRelationName = 'dummyGetLinkRel';
  const deleteLinkRel: LinkRelationName = 'dummyDeleteLinkRel';

  beforeEach(() => {
    config = {
      resource: configStateResource$,
      getLinkRel,
      edit: { linkRel: editLinkRel },
      delete: { linkRel: deleteLinkRel },
    };
    repository = mock(ResourceRepository);

    service = new ApiResourceService(config, useFromMock(repository));
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('save', () => {
    const dummyToSave: unknown = {};
    const loadedResource: Resource = createDummyResource();

    const resourceWithEditLinkRel: Resource = createDummyResource([editLinkRel]);

    it('should call repository', fakeAsync(() => {
      service.stateResource.next(createStateResource(resourceWithEditLinkRel));
      repository.save.mockReturnValue(of(loadedResource));

      service.save(dummyToSave).subscribe();
      tick();

      const expectedSaveResourceData: SaveResourceData<Resource> = {
        resource: resourceWithEditLinkRel,
        linkRel: editLinkRel,
        toSave: dummyToSave,
      };
      expect(repository.save).toHaveBeenCalledWith(expectedSaveResourceData);
    }));

    it('should return saved object', () => {
      service.stateResource.next(createStateResource(resourceWithEditLinkRel));
      repository.save.mockReturnValue(singleHot(loadedResource));

      const saved: Observable<StateResource<Resource | HttpError>> = service.save(dummyToSave);

      expect(saved).toBeObservable(singleCold(createStateResource(loadedResource)));
    });

    it('should call handleError', () => {
      service.stateResource.next(createStateResource(createDummyResource([config.edit.linkRel])));
      const errorResponse: ProblemDetail = createProblemDetail();
      repository.save.mockReturnValue(throwError(() => errorResponse));
      service.handleError = jest.fn();

      service.save(<any>{}).subscribe();

      expect(service.handleError).toHaveBeenCalledWith(errorResponse);
    });

    it('should update state resource subject', fakeAsync(() => {
      service.stateResource.next(createStateResource(resourceWithEditLinkRel));
      repository.save.mockReturnValue(of(loadedResource));

      service.save(dummyToSave).subscribe();
      tick();

      expect(service.stateResource.value).toEqual(createStateResource(loadedResource));
    }));
  });
});
