/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandOrder, CommandResource } from '@alfa-client/command-shared';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { cold } from 'jest-marbles';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createCommandResource, createCreateCommand } from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { Observable, of } from 'rxjs';
import { VorgangFacade } from './+state/vorgang.facade';
import { VorgangCommandService } from './vorgang-command.service';
import { VorgangWithEingangResource } from './vorgang.model';
import { VorgangService } from './vorgang.service';

describe('VorgangCommandService', () => {
  let service: VorgangCommandService;
  let facade: Mock<VorgangFacade>;
  let vorgangService: Mock<VorgangService>;

  const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

  beforeEach(() => {
    facade = mock(VorgangFacade);
    vorgangService = mock(VorgangService);

    service = new VorgangCommandService(useFromMock(facade), useFromMock(vorgangService));
  });

  describe('getAnnehmenCommand', () => {
    it('should call facade', () => {
      service.getAnnehmenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_ANNEHMEN);
    });
  });

  describe('annehmen', () => {
    it('should call facade', () => {
      service.annehmen(vorgangWithEingang);

      expect(facade.annehmen).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_ANNEHMEN),
      );
    });
  });

  describe('getBearbeitenCommand', () => {
    it('should call facade', () => {
      service.getBearbeitenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_BEARBEITEN);
    });
  });

  describe('bearbeiten', () => {
    it('should call facade', () => {
      service.bearbeiten(vorgangWithEingang);

      expect(facade.bearbeiten).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_BEARBEITEN),
      );
    });
  });

  describe('getBeschiedenCommand', () => {
    it('should call facade', () => {
      service.getBeschiedenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_BESCHEIDEN);
    });
  });

  describe('bescheiden', () => {
    const commandStateResource: StateResource<CommandResource> =
      createStateResource(createCommandResource());
    const commandStateResource$: Observable<StateResource<CommandResource>> =
      of(commandStateResource);

    beforeEach(() => {
      service.getBeschiedenCommand = jest.fn().mockReturnValue(commandStateResource$);
    });

    it('should call facade', () => {
      service.bescheiden(vorgangWithEingang);

      expect(facade.bescheiden).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_BESCHEIDEN),
      );
    });

    it('should call service do reload after', () => {
      service.doReloadAfter = jest.fn();

      service.bescheiden(vorgangWithEingang);

      expect(service.doReloadAfter).toHaveBeenCalledWith(commandStateResource$);
    });

    it('should return value', () => {
      const bescheiden$: Observable<StateResource<CommandResource>> =
        service.bescheiden(vorgangWithEingang);

      expect(bescheiden$).toBeObservable(cold('(a|)', { a: commandStateResource }));
    });
  });

  describe('getVerwerfenCommand', () => {
    it('should call facade', () => {
      service.getVerwerfenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_VERWERFEN);
    });
  });

  describe('verwerfen', () => {
    it('should call facade', () => {
      service.verwerfen(vorgangWithEingang);

      expect(facade.verwerfen).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_VERWERFEN),
      );
    });
  });

  describe('getWiedereroeffnenCommand', () => {
    it('should call facade', () => {
      service.getWiedereroeffnenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_WIEDEREROEFFNEN);
    });
  });

  describe('wiedereroeffnen', () => {
    it('should call facade', () => {
      service.wiedereroeffnen(vorgangWithEingang);

      expect(facade.wiedereroeffnen).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_WIEDEREROEFFNEN),
      );
    });
  });

  describe('getZurueckholenCommand', () => {
    it('should call facade', () => {
      service.getZurueckholenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_ZURUECKHOLEN);
    });
  });

  describe('zurueckholen', () => {
    it('should call facade', () => {
      service.zurueckholen(vorgangWithEingang);

      expect(facade.zurueckholen).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_ZURUECKHOLEN),
      );
    });
  });

  describe('getZurueckstellenCommand', () => {
    it('should call facade', () => {
      service.getZurueckstellenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_ZURUECKSTELLEN);
    });
  });

  describe('zurueckstellen', () => {
    it('should call facade', () => {
      service.zurueckstellen(vorgangWithEingang);

      expect(facade.zurueckstellen).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_ZURUECKSTELLEN),
      );
    });
  });

  describe('abschliessen', () => {
    const abschliessenCommand$: Observable<StateResource<CommandResource>> = of(
      createStateResource(createCommandResource()),
    );

    beforeEach(() => {
      service.getAbschliessenCommand = jest.fn().mockReturnValue(abschliessenCommand$);
    });

    it('should call facade', () => {
      service.abschliessen(vorgangWithEingang);

      expect(facade.abschliessen).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_ABSCHLIESSEN),
      );
    });

    it('should call service do reload after', () => {
      service.doReloadAfter = jest.fn();

      service.abschliessen(vorgangWithEingang);

      expect(service.doReloadAfter).toHaveBeenCalledWith(abschliessenCommand$);
    });

    it('should return value', () => {
      const abschliessenCommandStateResource: StateResource<CommandResource> =
        createStateResource(createCommandResource());
      service.getAbschliessenCommand = jest
        .fn()
        .mockReturnValue(of(abschliessenCommandStateResource));

      const abschliessen$ = service.abschliessen(vorgangWithEingang);

      expect(abschliessen$).toBeObservable(cold('(a|)', { a: abschliessenCommandStateResource }));
    });
  });

  describe('do reload after', () => {
    it('should reload vorgang', () => {
      const finishedCommand: CommandResource = createCommandResource([
        CommandLinkRel.EFFECTED_RESOURCE,
      ]);
      const commandStateResource$: Observable<StateResource<CommandResource>> = of(
        createStateResource(finishedCommand),
      );
      service.doReloadAfter(commandStateResource$).subscribe();

      expect(vorgangService.reloadVorgang).toHaveBeenCalledWith(finishedCommand);
    });

    it('should NOT reload vorgang on unfinished command', () => {
      const unfinishedCommand: CommandResource = createCommandResource();
      const commandStateResource$: Observable<StateResource<CommandResource>> = of(
        createStateResource(unfinishedCommand),
      );

      service.doReloadAfter(commandStateResource$).subscribe();

      expect(vorgangService.reloadVorgang).not.toHaveBeenCalled();
    });
  });

  describe('getAbschliessenCommand', () => {
    it('should call facade', () => {
      service.getAbschliessenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_ABSCHLIESSEN);
    });
  });

  describe('getRevokeCommand', () => {
    it('should call facade', () => {
      service.getRevokeCommand();

      expect(facade.getRevokeCommand).toHaveBeenCalled();
    });
  });
});
