/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ApiRootFacade, ApiRootLinkRel, ApiRootResource } from '@alfa-client/api-root-shared';
import {
  StateResource,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { cold, hot } from 'jest-marbles';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { createVorgangListResource } from 'libs/vorgang-shared/test/vorgang';
import { of } from 'rxjs';
import { VorgangFacade } from './+state/vorgang.facade';
import { VorgangListService } from './vorgang-list.service';
import { VorgangFilter, VorgangListResource, VorgangStatistic, VorgangView } from './vorgang.model';

import * as VorgangNavigationUtil from './vorgang-navigation.util';

describe('VorgangListService', () => {
  let service: VorgangListService;
  let vorgangFacade: Mock<VorgangFacade>;
  let apiRootFacade: Mock<ApiRootFacade>;

  beforeEach(() => {
    vorgangFacade = mock(VorgangFacade);
    apiRootFacade = mock(ApiRootFacade);

    service = new VorgangListService(useFromMock(vorgangFacade), useFromMock(apiRootFacade));
  });

  describe('getVorgangList', () => {
    describe('load required data', () => {
      const vorgangListStateResource: StateResource<VorgangListResource> = createStateResource(
        createVorgangListResource(),
      );

      beforeEach(() => {
        service.loadVorgangList = jest.fn();

        vorgangFacade.getVorgangList.mockReturnValue(of(vorgangListStateResource));
      });

      it('should get vorgang list by facade', () => {
        service.getVorgangList();

        expect(vorgangFacade.getVorgangList).toHaveBeenCalled();
      });

      it('should get apiroot by service', () => {
        service.getVorgangList();

        expect(apiRootFacade.getApiRoot).toHaveBeenCalled();
      });

      it('should get current route data by navigation facade', () => {
        service.getVorgangList();

        expect(apiRootFacade.getApiRoot).toHaveBeenCalled();
      });
    });

    describe('on loaded resource', () => {
      const vorgangListStateResource: StateResource<VorgangListResource> = createStateResource(
        createVorgangListResource(),
      );
      const apiRootStateResource: StateResource<ApiRootResource> =
        createStateResource(createApiRootResource());
      const vorgangFilter: VorgangFilter = VorgangFilter.ALLE;
      const vorgangView: VorgangView = VorgangView.NEU;
      const searchString: string = 'dummySearchString';

      beforeEach(() => {
        service.loadVorgangList = jest.fn();

        vorgangFacade.getVorgangList.mockReturnValue(hot('-a', { a: vorgangListStateResource }));
        apiRootFacade.getApiRoot.mockReturnValue(of(apiRootStateResource));
        vorgangFacade.getVorgangFilter.mockReturnValue(of(vorgangFilter));
        vorgangFacade.getVorgangView.mockReturnValue(of(vorgangView));
        vorgangFacade.getSearchString.mockReturnValue(of(searchString));
      });

      it('should return loaded resource', () => {
        const vorgangList = service.getVorgangList();

        expect(vorgangList).toBeObservable(
          cold('ab', { a: createEmptyStateResource(true), b: vorgangListStateResource }),
        );
      });

      it('should not load vorgangList', () => {
        service.getVorgangList();

        expect(service.loadVorgangList).not.toHaveBeenCalled();
      });
    });

    describe('on reload required', () => {
      const vorgangListResource: VorgangListResource = createVorgangListResource();
      const vorgangListStateResource: StateResource<VorgangListResource> = {
        ...createStateResource(vorgangListResource),
        reload: true,
      };
      const apiRootStateResource: StateResource<ApiRootResource> =
        createStateResource(createApiRootResource());
      const vorgangFilter: VorgangFilter = VorgangFilter.ALLE;
      const vorgangView: VorgangView = VorgangView.NEU;
      const searchString: string = 'dummySearchString';

      beforeEach(() => {
        vorgangFacade.getVorgangList.mockReturnValue(hot('-a', { a: vorgangListStateResource }));
        apiRootFacade.getApiRoot.mockReturnValue(of(apiRootStateResource));
        vorgangFacade.getVorgangFilter.mockReturnValue(of(vorgangFilter));
        vorgangFacade.getVorgangView.mockReturnValue(of(vorgangView));
        vorgangFacade.getSearchString.mockReturnValue(of(searchString));
      });

      it('should return value on loaded resource', () => {
        const vorgangList = service.getVorgangList();

        expect(vorgangList).toBeObservable(
          cold('ab', { a: createEmptyStateResource(true), b: vorgangListStateResource }),
        );
      });
    });
  });

  describe('loadVorgangList', () => {
    const apiRootResource: ApiRootResource = createApiRootResource([ApiRootLinkRel.SEARCH_ALLE]);
    const vorgangFilter: VorgangFilter = VorgangFilter.ALLE;
    const searchString: string = 'searchDummyString';

    describe('on search view', () => {
      const vorgangView: VorgangView = VorgangView.SEARCH;

      it('should call facade searchVorgaengeBy', () => {
        service.loadVorgangList(apiRootResource, vorgangFilter, vorgangView, searchString);

        expect(vorgangFacade.searchVorgaengeBy).toHaveBeenCalledWith(
          apiRootResource,
          searchString,
          ApiRootLinkRel.SEARCH_ALLE,
        );
      });

      it('should call getSearchLinkRel', () => {
        const utilSpy = jest.spyOn(VorgangNavigationUtil, 'getSearchLinkRel');

        service.loadVorgangList(apiRootResource, vorgangFilter, vorgangView, searchString);

        expect(utilSpy).toHaveBeenCalledWith(vorgangFilter);
      });
    });

    describe('on vorganglist loadVorgangList', () => {
      const vorgangView: VorgangView = VorgangView.VORGANG_LIST;

      it('should call facade loadVorgaengeList', () => {
        service.loadVorgangList(apiRootResource, vorgangFilter, vorgangView, searchString);

        expect(vorgangFacade.loadVorgangList).toHaveBeenCalled();
      });

      it('should call buildLinkRel', () => {
        const utilSpy = jest.spyOn(VorgangNavigationUtil, 'buildLinkRel');

        service.loadVorgangList(apiRootResource, vorgangFilter, vorgangView, searchString);

        expect(utilSpy).toHaveBeenCalledWith(vorgangFilter, vorgangView);
      });
    });
  });

  describe('loadNextPage', () => {
    it('should call facade', () => {
      service.loadNextPage();

      expect(vorgangFacade.loadNextPage).toHaveBeenCalled();
    });
  });

  describe('getVorgaenge', () => {
    it('should call facade', () => {
      service.getVorgaenge();

      expect(vorgangFacade.getVorgaenge).toHaveBeenCalled();
    });
  });

  describe('getVorgangStatistic', () => {
    const vorgangStatisticStateResource: StateResource<VorgangStatistic> = createStateResource(
      <VorgangStatistic>{},
    );

    beforeEach(() => {
      vorgangFacade.getVorgangStatistic.mockReturnValue(of(vorgangStatisticStateResource));
      service.getVorgangList = jest
        .fn()
        .mockReturnValue(of(<StateResource<VorgangListResource>>{}));
    });

    it('should call facade', () => {
      service.getVorgangStatistic();

      expect(vorgangFacade.getVorgangStatistic).toHaveBeenCalled();
    });

    it('should return vorgangStatistic', (done) => {
      service.getVorgangStatistic().subscribe((vorgangStatistic) => {
        expect(vorgangStatistic).toEqual(vorgangStatisticStateResource);
        done();
      });
    });
  });

  describe('getSelectedFilterAsRouteParam', () => {
    beforeEach(() => {
      vorgangFacade.getVorgangFilter.mockReturnValue(of(VorgangFilter.ALLE));
    });

    it('should call facade', (done) => {
      service.getSelectedFilterAsRouteParam().subscribe(() => {
        expect(vorgangFacade.getVorgangFilter).toHaveBeenCalled();
        done();
      });
    });

    it('should return value', (done) => {
      service.getSelectedFilterAsRouteParam().subscribe((routeParam) => {
        expect(routeParam).toBe('alle');
        done();
      });
    });
  });

  describe('isVorgangViewSelected', () => {
    const vorgangView: VorgangView = VorgangView.VORGANG_LIST;

    it('should call facade', () => {
      service.isVorgangViewSelected(vorgangView);

      expect(vorgangFacade.isVorgangViewSelected).toHaveBeenCalledWith(vorgangView);
    });
  });

  describe('getVorgangViewRoutePath', () => {
    const vorgangView: VorgangView = VorgangView.VORGANG_LIST;

    it('should call facade', () => {
      service.getVorgangViewRoutePath(vorgangView);

      expect(vorgangFacade.getVorgangViewRoutePath).toHaveBeenCalledWith(vorgangView);
    });
  });

  describe('getSearchPreviewList', () => {
    describe('load required data', () => {
      const vorgangListStateResource: StateResource<VorgangListResource> = createStateResource(
        createVorgangListResource(),
      );

      beforeEach(() => {
        service.loadVorgangList = jest.fn();

        vorgangFacade.getSearchPreviewList.mockReturnValue(of(vorgangListStateResource));
      });

      it('should get apiroot by facade', () => {
        service.getSearchPreviewList();

        expect(apiRootFacade.getApiRoot).toHaveBeenCalled();
      });

      it('should get search preview list by facade', () => {
        service.getSearchPreviewList();

        expect(vorgangFacade.getSearchPreviewList).toHaveBeenCalled();
      });

      it('should get search string by facade', () => {
        service.getSearchPreviewList();

        expect(vorgangFacade.getSearchString).toHaveBeenCalled();
      });
    });

    describe('on loaded resource', () => {
      const vorgangListStateResource: StateResource<VorgangListResource> = createStateResource(
        createVorgangListResource(),
      );
      const apiRootStateResource: StateResource<ApiRootResource> =
        createStateResource(createApiRootResource());
      const searchString: string = 'dummySearchString';
      const vorgangFilter: VorgangFilter = VorgangFilter.ALLE;

      beforeEach(() => {
        vorgangFacade.getSearchPreviewList.mockReturnValue(
          hot('-a', { a: vorgangListStateResource }),
        );
        vorgangFacade.getSearchString.mockReturnValue(of(searchString));
        apiRootFacade.getApiRoot.mockReturnValue(of(apiRootStateResource));
        vorgangFacade.getVorgangFilter.mockReturnValue(of(vorgangFilter));
      });

      it('should return loaded resource', () => {
        const searchPreviewList = service.getSearchPreviewList();

        expect(searchPreviewList).toBeObservable(
          cold('ab', { a: createEmptyStateResource(true), b: vorgangListStateResource }),
        );
      });

      it('should not load searchPreviewList', () => {
        service.getSearchPreviewList();

        expect(vorgangFacade.searchForPreview).not.toHaveBeenCalled();
      });
    });

    describe('on reload required', () => {
      const searchPreviewList: StateResource<VorgangListResource> = {
        ...createStateResource(createVorgangListResource()),
        reload: true,
      };
      const apiRootStateResource: StateResource<ApiRootResource> =
        createStateResource(createApiRootResource());
      const searchString: string = 'dummySearchString';
      const vorgangFilter: VorgangFilter = VorgangFilter.ALLE;

      beforeEach(() => {
        vorgangFacade.getSearchPreviewList.mockReturnValue(hot('-a', { a: searchPreviewList }));
        vorgangFacade.getSearchString.mockReturnValue(of(searchString));
        apiRootFacade.getApiRoot.mockReturnValue(of(apiRootStateResource));
        vorgangFacade.getVorgangFilter.mockReturnValue(of(vorgangFilter));
      });

      it('should return value on loaded resource', () => {
        const vorgangList = service.getSearchPreviewList();

        expect(vorgangList).toBeObservable(
          cold('ab', {
            a: createEmptyStateResource(true),
            b: { ...searchPreviewList, reload: true },
          }),
        );
      });

      it('should call facade searchForPreview', () => {
        vorgangFacade.getSearchPreviewList.mockReturnValue(of(searchPreviewList));

        service.getSearchPreviewList().subscribe();

        expect(vorgangFacade.searchForPreview).toHaveBeenCalledWith(searchString);
      });
    });
  });

  describe('clearSearchPreviewList', () => {
    it('should call facade clearSearchPreviewList', () => {
      service.clearSearchPreviewList();

      expect(vorgangFacade.clearSearchPreviewList).toHaveBeenCalled();
    });
  });

  describe('clearSearchString', () => {
    it('should call facade clearSearchString', () => {
      service.clearSearchString();

      expect(vorgangFacade.clearSearchString).toHaveBeenCalled();
    });
  });

  describe('getSearchString', () => {
    it('should call facade getSearchString', () => {
      service.getSearchString();

      expect(vorgangFacade.getSearchString).toHaveBeenCalled();
    });
  });

  describe('searchForPreview', () => {
    it('should call facade searchForPreview', () => {
      const searchString: string = 'dummySearchString';
      service.searchForPreview(searchString);

      expect(vorgangFacade.searchForPreview).toHaveBeenCalledWith(searchString);
    });
  });
});
