package de.ozgcloud.alfa;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.ModelBuilder;
import de.ozgcloud.alfa.common.downloadtoken.DownloadTokenController;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;
import de.ozgcloud.alfa.common.user.UserRole;

@Component
public class RootModelAssembler implements RepresentationModelAssembler<Root, EntityModel<Root>> {

	static final String REL_DOWNLOAD_TOKEN = "downloadToken";
	static final String REL_CURRENT_USER = "currentUser";
	static final String REL_DOCUMENTATIONS = "documentations";

	@Autowired
	private RootViewLinkHandler viewLinkHandler;
	@Autowired
	private CurrentUserService currentUserService;
	@Autowired
	private UserManagerUrlProvider userManagerUrlProvider;

	@Value("${ozgcloud.user-assistance.documentation.url:#{null}}")
	private String documentationUrl;

	@Override
	public EntityModel<Root> toModel(Root root) {
		var modelBuilder = ModelBuilder.fromEntity(root)
				.ifMatch(this::hasAnyRole).addLinks(
						linkTo(RootController.class).withSelfRel(),
						linkTo(DownloadTokenController.class).withRel(REL_DOWNLOAD_TOKEN))
				.ifMatch(userManagerUrlProvider::isConfiguredForUserProfile)
				.addLinkIfPresent(this::buildCurrentUserLink)
				.addLink(buildUserAssistanceDocumentationUrl());

		if (hasAnyRole()) {
			viewLinkHandler.addViewLinks(modelBuilder, currentUserService.findUserId());
		}

		return modelBuilder.buildModel();
	}

	boolean hasAnyRole() {
		return hasVerwaltungRole() || currentUserService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER);
	}

	boolean hasVerwaltungRole() {
		return currentUserService.hasRole(UserRole.VERWALTUNG_USER) || currentUserService.hasRole(UserRole.VERWALTUNG_POSTSTELLE);
	}

	private Optional<Link> buildCurrentUserLink() {
		return currentUserService.findUserId().map(userId -> Link.of(String.format(userManagerUrlProvider.getUserProfileTemplate(), userId))
				.withRel(REL_CURRENT_USER));
	}

	private Optional<Link> buildUserAssistanceDocumentationUrl() {
		return Optional.ofNullable(documentationUrl).filter(StringUtils::isNotBlank).map(href -> Link.of(href, REL_DOCUMENTATIONS));
	}
}