package de.ozgcloud.alfa.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.Optional;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.SneakyThrows;

class BescheidControllerTest {

	@Spy
	@InjectMocks
	private BescheidController controller;
	@Mock
	private BescheidModelAssembler assembler;
	@Mock
	private BescheidService bescheidService;
	@Mock
	private BinaryFileController binaryFileController;

	private MockMvc mockMvc;

	private final Bescheid draft = BescheidTestFactory.create();

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetDraft {

		@SneakyThrows
		@Test
		void shouldCallBescheidService() {
			doRequest();

			verify(bescheidService).getBescheidDraft(VorgangHeaderTestFactory.ID);
		}

		@SneakyThrows
		@Test
		void shouldReturnStatusNotFoundIfDraftDoesNotExist() {
			when(bescheidService.getBescheidDraft(BescheidTestFactory.VORGANG_ID)).thenReturn(Optional.empty());

			var response = doRequest();

			response.andExpect(status().isNotFound());
		}

		@DisplayName("on existing")
		@Nested
		class TestOnExistingDraft {

			@BeforeEach
			void mock() {
				when(bescheidService.getBescheidDraft(BescheidTestFactory.VORGANG_ID)).thenReturn(Optional.of(draft));
			}

			@SneakyThrows
			@Test
			void shouldCallAssembler() {
				doRequest();

				verify(assembler).toModel(draft);
			}

			@SneakyThrows
			@Test
			void shouldReturnValue() {
				when(assembler.toModel(draft)).thenReturn(EntityModel.of(draft));

				doRequest().andExpect(status().isOk());
			}
		}

		@SneakyThrows
		private ResultActions doRequest() {
			return mockMvc.perform(get(BescheidController.PATH).param("vorgangId", VorgangHeaderTestFactory.ID));
		}
	}

	@DisplayName("Get Attachments")
	@Nested
	class TestGetAttachments {

		@BeforeEach
		void mock() {
			doReturn(BescheidTestFactory.create()).when(controller).getBescheid(anyString(), anyString());
		}

		@SneakyThrows
		@Test
		void shouldGetBescheid() {
			doRequest();

			verify(controller).getBescheid(BescheidTestFactory.ID, BescheidTestFactory.VORGANG_ID);
		}

		@SneakyThrows
		@Test
		void shouldCallBinaryFileController() {
			doRequest();

			verify(binaryFileController).getFiles(BescheidTestFactory.ATTACHMENTS);
		}

		@SneakyThrows
		@Test
		void shouldReturnStatusOk() {
			var response = doRequest();

			response.andExpect(status().isOk());
		}

		@SneakyThrows
		private ResultActions doRequest() {
			return mockMvc.perform(
					get(BescheidController.PATH + "/{bescheidId}/attachments", BescheidTestFactory.ID).param("vorgangId",
							BescheidTestFactory.VORGANG_ID));
		}
	}

	@DisplayName("Get Bescheid")
	@Nested
	class TestGetBescheid {

		private final Bescheid matchingBescheid = BescheidTestFactory.create();
		private final Bescheid notMatchingBescheid = BescheidTestFactory.createBuilder().id("NOT_MATCH").build();

		@BeforeEach
		void mock() {
			when(bescheidService.findByVorgangId(anyString())).thenReturn(Stream.of(matchingBescheid, notMatchingBescheid));
		}

		@DisplayName("should call service to find all bescheid by vorgang")
		@Test
		void shouldCallFindByVorgangId() {
			controller.getBescheid(BescheidTestFactory.ID, VorgangHeaderTestFactory.ID);

			verify(bescheidService).findByVorgangId(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldReturnMatchingEntry() {
			var bescheid = controller.getBescheid(BescheidTestFactory.ID, VorgangHeaderTestFactory.ID);

			assertThat(bescheid).isEqualTo(matchingBescheid);
		}

		@Test
		void shouldThrowExceptionIfNoBescheidExists() {
			when(bescheidService.findByVorgangId(anyString())).thenReturn(Stream.empty());

			assertThatThrownBy(() -> controller.getBescheid(BescheidTestFactory.ID, VorgangHeaderTestFactory.ID))
					.isInstanceOf(TechnicalException.class)
					.hasMessageContaining("No Bescheid exists for id " + BescheidTestFactory.ID);
		}
	}
}