package de.ozgcloud.alfa.common;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.ZonedDateTime;

import org.apache.commons.io.FilenameUtils;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.xoev.xdomea.DateiformatCodeType;
import de.xoev.xdomea.FormatType;

class VersionTypeBuilderTest {
	private static final String FILE_NAME = "TestFile.ext";
	private final OzgFile ozgFile = OzgFileTestFactory.createBuilder().name(FILE_NAME).build();
	private final static ZonedDateTime CREATED_AT = ZonedDateTime.now();
	private final static String ERSTELLER = LoremIpsum.getInstance().getName();
	private final static String SONSTIGER_NAME = LoremIpsum.getInstance().getName();
	@Spy
	private VersionTypeBuilder builder = VersionTypeBuilder.builder().withOzgFile(ozgFile).withCreatedAt(CREATED_AT).withErsteller(ERSTELLER)
			.withSonstigerName(SONSTIGER_NAME);

	@Nested
	class TestBuild {
		private final FormatType formatType = FormatTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			doReturn(formatType).when(builder).createFormatType();
		}

		@Test
		void shouldCallCreateFormatType() {
			builder.build();

			verify(builder).createFormatType();
		}

		@Test
		void shouldHaveNummer() {
			var version = builder.build();

			assertThat(version.getNummer()).isEqualTo(VersionTypeBuilder.VERSION_NUMMER);
		}

		@Test
		void shouldHaveFormatType() {
			var version = builder.build();

			assertThat(version.getFormat()).contains(formatType);
		}
	}

	@Nested
	class TestCreateFormatType {
		private MockedStatic<PrimaerdokumentTypeBuilder> primaerdokumentTypeBuilderMockedStatic;
		@Mock
		private PrimaerdokumentTypeBuilder primaerdokumentTypeBuilder;

		private final DateiformatCodeType dateiformatCodeType = DateiformatCodeTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			primaerdokumentTypeBuilderMockedStatic = mockStatic(PrimaerdokumentTypeBuilder.class);
			primaerdokumentTypeBuilderMockedStatic.when(PrimaerdokumentTypeBuilder::builder).thenReturn(primaerdokumentTypeBuilder);
			when(primaerdokumentTypeBuilder.withOzgFile(ozgFile)).thenReturn(primaerdokumentTypeBuilder);
			when(primaerdokumentTypeBuilder.withCreatedAt(CREATED_AT)).thenReturn(primaerdokumentTypeBuilder);
			when(primaerdokumentTypeBuilder.withErsteller(ERSTELLER)).thenReturn(primaerdokumentTypeBuilder);
			doReturn(dateiformatCodeType).when(builder).createDateiformatCodeType();
		}

		@AfterEach
		void tearDown() {
			primaerdokumentTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldBuildPrimaerdokument() {
			callCreateFormatType();

			verify(primaerdokumentTypeBuilder).build();
		}

		@Test
		void shouldSetOzgFile() {
			callCreateFormatType();

			verify(primaerdokumentTypeBuilder).withOzgFile(ozgFile);
		}

		@Test
		void shouldSetCreatedAt() {
			callCreateFormatType();

			verify(primaerdokumentTypeBuilder).withCreatedAt(CREATED_AT);
		}

		@Test
		void shouldSetErsteller() {
			callCreateFormatType();

			verify(primaerdokumentTypeBuilder).withErsteller(ERSTELLER);
		}

		@Test
		void shouldHaveEmptyVersion() {
			var formatType = callCreateFormatType();

			assertThat(formatType.getVersion()).isEmpty();
		}

		@Test
		void shouldCreateDateiformatCode() {
			callCreateFormatType();

			verify(builder).createDateiformatCodeType();
		}

		@Test
		void shouldHaveName() {
			var formatType = callCreateFormatType();

			assertThat(formatType.getName()).isEqualTo(dateiformatCodeType);
		}

		@Test
		void shouldHaveSonstigerName() {
			var formatType = callCreateFormatType();

			assertThat(formatType.getSonstigerName()).isEqualTo(SONSTIGER_NAME);
		}

		private FormatType callCreateFormatType() {
			return builder.createFormatType();
		}
	}

	@Nested
	class TestCreateDateiformatCodeType {

		private static final String CODE = "111";

		private MockedStatic<DateiformatCode> dateiformatCodeMockedStatic;

		@BeforeEach
		void setUp() {
			dateiformatCodeMockedStatic = mockStatic(DateiformatCode.class);
			dateiformatCodeMockedStatic.when(
					() -> DateiformatCode.getXdomeaCode(OzgFileTestFactory.CONTENT_TYPE, FilenameUtils.getExtension(FILE_NAME)))
					.thenReturn(CODE);
		}

		@AfterEach
		void tearDown() {
			dateiformatCodeMockedStatic.close();
		}

		@Test
		void shouldHaveListURI() {
			var dateiformatCode = builder.createDateiformatCodeType();

			assertThat(dateiformatCode.getListURI()).isEqualTo(VersionTypeBuilder.DATEI_FORMAT_LIST_URI);
		}

		@Test
		void shouldHaveListVersionID() {
			var dateiformatCode = builder.createDateiformatCodeType();

			assertThat(dateiformatCode.getListVersionID()).isEqualTo(VersionTypeBuilder.LIST_VERSION_ID);
		}

		@Test
		void shouldGetXdomeaCode() {
			builder.createDateiformatCodeType();

			dateiformatCodeMockedStatic.verify(
					() -> DateiformatCode.getXdomeaCode(OzgFileTestFactory.CONTENT_TYPE, FilenameUtils.getExtension(FILE_NAME)));
		}

		@Test
		void shouldTransformExtensionToLowerCase() {
			builder.withOzgFile(OzgFileTestFactory.createBuilder().name("TestFile.EXT").build());

			builder.createDateiformatCodeType();

			dateiformatCodeMockedStatic.verify(
					() -> DateiformatCode.getXdomeaCode(OzgFileTestFactory.CONTENT_TYPE, FilenameUtils.getExtension(FILE_NAME)));
		}

		@Test
		void shouldHaveCode() {
			var dateiformatCode = builder.createDateiformatCodeType();

			assertThat(dateiformatCode.getCode()).isEqualTo(CODE);
		}
	}
}
