package de.ozgcloud.alfa.file;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Random;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.MockedStatic;

import de.ozgcloud.alfa.common.UUIDConverter;
import de.ozgcloud.alfa.common.VersionTypeBuilder;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.DateiformatCodeType;
import de.xoev.xdomea.FormatType;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.PrimaerdokumentType;
import de.xoev.xdomea.VersionType;

class DokumentTypeBuilderITCase {

	private static final Long DOKUMENT_ORDINAL_NUMBER = new Random().nextLong();
	private static final String FORM_ENGINE_NAME = "DUMMY_NAME";
	private final OzgFile ozgFile = OzgFileTestFactory.createBuilder().contentType("application/pdf").build();

	private final DokumentTypeBuilder builder = DokumentTypeBuilder.builder()
			.withOzgFile(ozgFile)
			.withFormEngineName(FORM_ENGINE_NAME)
			.withOrdinalNumber(DOKUMENT_ORDINAL_NUMBER);

	private MockedStatic<UUIDConverter> uuidConverter;

	@BeforeEach
	void init() {
		uuidConverter = mockStatic(UUIDConverter.class);
	}

	@AfterEach
	void cleanup() {
		uuidConverter.close();
	}

	@Nested
	class TestBuild {

		private final String FILE_UUID = "64a820d3-6285-172a-c028-0000000026d0";

		@BeforeEach
		void init() {
			uuidConverter.when(() -> UUIDConverter.fromObjectId(OzgFileTestFactory.ID.toString())).thenReturn(FILE_UUID);
		}

		@Test
		void shouldHaveIdentifikation() {
			var identifikationObjekt = new IdentifikationObjektType();
			identifikationObjekt.setID(FILE_UUID);
			identifikationObjekt.setNummerImUebergeordnetenContainer(DOKUMENT_ORDINAL_NUMBER);
			var dokument = builder.build();

			assertThat(dokument.getIdentifikation()).usingRecursiveComparison().isEqualTo(identifikationObjekt);
		}

		@Nested
		class TestAllgemeineMetadaten {
			@Test
			void shouldHaveBetreff() {
				var allgemeineMetadaten = createAllgemeineMetadaten();

				assertThat(allgemeineMetadaten.getBetreff()).isEqualTo(OzgFileTestFactory.NAME);
			}

			@Test
			void shouldHaveEmptyKennzeichen() {
				var allgemeineMetadaten = createAllgemeineMetadaten();

				assertThat(allgemeineMetadaten.getKennzeichen()).isEmpty();
			}

			@Test
			void shouldHaveEmptyBemerkung() {
				var allgemeineMetadaten = createAllgemeineMetadaten();

				assertThat(allgemeineMetadaten.getBemerkung()).isEmpty();
			}

			@Test
			void shouldHaveMedium() {
				var allgemeineMetadaten = createAllgemeineMetadaten();

				assertThat(allgemeineMetadaten.getMedium().getCode()).isEqualTo(DokumentTypeBuilder.ALLGEMEINE_METADATEN_MEDIUM_CODE);
			}

			private AllgemeineMetadatenType createAllgemeineMetadaten() {
				return builder.build().getAllgemeineMetadaten();
			}
		}

		@Nested
		class TestVersion {

			@Test
			void shouldHaveNummer() {
				var versionType = createVersion();

				assertThat(versionType.getNummer()).isEqualTo(DokumentTypeBuilder.VERSION_NUMMER);
			}

			private VersionType createVersion() {
				return builder.build().getVersion().getFirst();
			}

			@Nested
			class TestFormat {
				@Test
				void shouldHaveName() {
					var expectedDateiformatCode = createDateiformatCode("018");

					var formatType = createFormatType();

					assertThat(formatType.getName()).usingRecursiveComparison().isEqualTo(expectedDateiformatCode);
				}

				private DateiformatCodeType createDateiformatCode(String codeValue) {
					var dateiFormatCode = new DateiformatCodeType();
					dateiFormatCode.setCode(codeValue);
					dateiFormatCode.setListURI(VersionTypeBuilder.DATEI_FORMAT_LIST_URI);
					dateiFormatCode.setListVersionID(VersionTypeBuilder.LIST_VERSION_ID);
					return dateiFormatCode;
				}

				@Test
				void shouldHaveSonstinerNameEmptyString() {
					var formatType = createFormatType();

					assertThat(formatType.getSonstigerName()).isEqualTo(StringUtils.EMPTY);
				}

				@Test
				void shouldHaveVersionEmptyString() {
					var formatType = createFormatType();

					assertThat(formatType.getVersion()).isEqualTo(StringUtils.EMPTY);
				}

				private FormatType createFormatType() {
					return createVersion().getFormat().getFirst();
				}

				@Nested
				class TestPrimaerDokument {

					@Test
					void shouldHaveDateiname() {
						var primaerdokument = createPrimaerdokumentType();

						assertThat(primaerdokument.getDateiname()).isEqualTo(FILE_UUID + "_" + ozgFile.getName());
					}

					@Test
					void shouldHaveDateinameOriginal() {
						var primaerdokument = createPrimaerdokumentType();

						assertThat(primaerdokument.getDateinameOriginal()).isEqualTo(ozgFile.getName());
					}

					@Test
					void shouldHaveErsteller() {
						var primaerdokument = createPrimaerdokumentType();

						assertThat(primaerdokument.getErsteller()).isEqualTo(FORM_ENGINE_NAME);
					}

					private PrimaerdokumentType createPrimaerdokumentType() {
						return createFormatType().getPrimaerdokument();
					}
				}

			}

		}
	}

}