package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import javax.xml.datatype.XMLGregorianCalendar;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.DateConverter;
import de.xoev.xdomea.GeburtType;
import de.xoev.xdomea.KontaktType;
import de.xoev.xdomea.NameNatuerlichePersonType;
import jakarta.annotation.Nullable;

class KontaktTypeCreatorTest {

	@Spy
	@InjectMocks
	private KontaktTypeCreator creator;

	@Mock
	private NameNatuerlichePersonTypeCreator nameNatuerlichePersonTypeCreator;

	@Nested
	class TestCreate {

		@Nested
		class TestWithAntragsteller {

			private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
			private final KontaktType kontaktType = KontaktTypeTestFactory.create();

			@BeforeEach
			void setUp() {
				doReturn(Optional.of(vorgang.getEingang().getAntragsteller())).when(creator).getAntragstellerIfHasRequiredData(vorgang);
				doReturn(kontaktType).when(creator).toKontaktType(vorgang.getEingang().getAntragsteller());
			}

			@Test
			void shouldGetAntragsteller() {
				callCreator(vorgang);

				verify(creator).getAntragstellerIfHasRequiredData(vorgang);
			}

			@Test
			void shouldMapToKontaktType() {
				callCreator(vorgang);

				verify(creator).toKontaktType(vorgang.getEingang().getAntragsteller());
			}

			@Test
			void shouldReturnKontakt() {
				var kontakt = callCreator(vorgang);

				assertThat(kontakt).get().isEqualTo(kontaktType);
			}
		}

		@Nested
		class TestWithoutAntragsteller {

			private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.createBuilder()
					.eingang(EingangTestFactory.createBuilder().antragsteller(null).build())
					.build();

			@BeforeEach
			void setUp() {
				doReturn(Optional.empty()).when(creator).getAntragstellerIfHasRequiredData(vorgang);
			}

			@Test
			void shouldBeEmpty() {
				var kontakt = callCreator(vorgang);

				assertThat(kontakt).isEmpty();
			}
		}

		private Optional<KontaktType> callCreator(VorgangWithEingang vorgang) {
			return creator.create(vorgang);
		}
	}

	@Nested
	class TestGeburtsdatumToISO {

		@Mock
		private XMLGregorianCalendar geburtsdatum;

		private MockedStatic<DateConverter> dateConverter;

		private final Antragsteller antragsteller = AntragstellerTestFactory.create();

		void setUpDateConverter() {
			dateConverter = mockStatic(DateConverter.class);
			dateConverter.when(() -> DateConverter.convertGermanFormatToISO(AntragstellerTestFactory.GEBURTSDATUM_STR))
					.thenReturn(Optional.of(geburtsdatum));
		}

		@BeforeEach
		void setUp() {
			setUpDateConverter();
		}

		@AfterEach
		void cleanUp() {
			dateConverter.close();
		}

		@Test
		void shouldCallDateConverter() {
			callGeburtsdatumToISO(antragsteller);

			dateConverter.verify(() -> DateConverter.convertGermanFormatToISO(antragsteller.getGeburtsdatum()));
		}

		@Test
		void shouldReturnGeburtsdatumIso() {
			var result = callGeburtsdatumToISO(antragsteller);

			assertThat(result).isEqualTo(Optional.of(geburtsdatum));
		}

		private Optional<XMLGregorianCalendar> callGeburtsdatumToISO(Antragsteller antragsteller) {
			return creator.geburtsdatumToISO(antragsteller);
		}

	}

	@Nested
	class TestToKontaktType {

		@Nested
		class WhenHavingGeburtsdatum {

			@Mock
			private XMLGregorianCalendar geburtsdatum;

			private MockedStatic<KontaktTypeCreator> mockStatic;

			private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
			private final Antragsteller antragsteller = vorgang.getEingang().getAntragsteller();
			private final NameNatuerlichePersonType nameNatuerlichePerson = NameNatuerlichePersonTypeTestFactory.create();
			private final GeburtType geburtType = GeburtTypeTestFactory.create();

			@BeforeEach
			void setUp() {
				when(nameNatuerlichePersonTypeCreator.create(vorgang.getEingang().getAntragsteller())).thenReturn(nameNatuerlichePerson);

				mockStatic = mockStatic(KontaktTypeCreator.class);
				mockStatic.when(() -> KontaktTypeCreator.geburtsdatumToISO(antragsteller)).thenReturn(Optional.of(geburtsdatum));

				doReturn(geburtType).when(creator).createGeburtType(geburtsdatum);
			}

			@AfterEach
			void tearDown() {
				mockStatic.close();
			}

			@Test
			void shouldCreateNameNatuerlichePerson() {
				callCreator(vorgang);

				verify(nameNatuerlichePersonTypeCreator).create(vorgang.getEingang().getAntragsteller());
			}

			@Test
			void shouldHaveNameNatuerlichePerson() {
				var kontakt = callCreator(vorgang);

				assertThat(kontakt.getName()).isEqualTo(nameNatuerlichePerson);
			}

			@Test
			void shouldCallDateConverter() {
				callCreator(vorgang);

				verify(creator).createGeburtType(geburtsdatum);
			}

			@Test
			void shouldCreateGeburtType() {
				callCreator(vorgang);

				verify(creator).createGeburtType(geburtsdatum);
			}

			@Test
			void shouldHaveGeburtType() {
				var kontakt = callCreator(vorgang);

				assertThat(kontakt.getGeburt()).isEqualTo(geburtType);
			}
		}

		@Nested
		class WhenHavingInvalidGeburtsdatum {

			@ParameterizedTest
			@NullAndEmptySource
			@ValueSource(strings = { "unreadable" })
			void shouldNotSetForInvalidGeburtsdatum(String geburtsdatum) {
				var result = callCreator(createVorgangWithAntragstellerGeburtsdatum(geburtsdatum));

				assertThat(result.getGeburt()).isNull();
			}

			@ParameterizedTest
			@NullAndEmptySource
			@ValueSource(strings = { "unreadable" })
			void shouldNotCallCreateGeburtType(String geburtsdatum) {
				callCreator(createVorgangWithAntragstellerGeburtsdatum(geburtsdatum));

				verify(creator, never()).createGeburtType(any());
			}

		}

		private VorgangWithEingang createVorgangWithAntragstellerGeburtsdatum(@Nullable String geburtsdatum) {
			return VorgangWithEingangTestFactory.createBuilder().eingang(
					EingangTestFactory.createBuilder().antragsteller(
							AntragstellerTestFactory.createBuilder().geburtsdatum(geburtsdatum).build()).build()).build();
		}

		private KontaktType callCreator(VorgangWithEingang vorgang) {
			return creator.toKontaktType(vorgang.getEingang().getAntragsteller());
		}
	}

	@Nested
	class TestCreateGeburtType {

		@Mock
		private XMLGregorianCalendar geburtsdatum;

		private MockedStatic<DateConverter> dateConverter;

		@BeforeEach
		void setUp() {
			dateConverter = mockStatic(DateConverter.class);
			dateConverter.when(() -> DateConverter.convertGermanFormatToISO(AntragstellerTestFactory.GEBURTSDATUM_STR))
					.thenReturn(Optional.of(geburtsdatum));
		}

		@AfterEach
		void tearDown() {
			dateConverter.close();
		}

		@Test
		void shouldCreate() {
			var geburtType = callCreator(geburtsdatum);

			assertThat(geburtType.getDatum()).isEqualTo(geburtsdatum);
		}

		private GeburtType callCreator(XMLGregorianCalendar geburtsdatum) {
			return creator.createGeburtType(geburtsdatum);
		}
	}

	@Nested
	class TestGetAntragstellerIfHasRequiredData {

		@Mock
		private XMLGregorianCalendar geburtsdatum;

		@Test
		void shouldBeEmptyIfAntragstellerNull() {
			var antragsteller = callCreator(buildVorgang(null));

			assertThat(antragsteller).isEmpty();
		}

		@Test
		void shouldBeEmptyIfNoneOfRequiredDataExists() {
			try (var dateConverter = mockStatic(DateConverter.class)) {
				dateConverter.when(() -> DateConverter.convertGermanFormatToISO(null)).thenReturn(Optional.empty());

				var antragsteller = callCreator(buildVorgang(Antragsteller.builder().build()));

				assertThat(antragsteller).isEmpty();
			}
		}

		@Test
		void shouldNotBeEmptyIfHasVorname() {
			var antragsteller = callCreator(buildVorgang(Antragsteller.builder().vorname(AntragstellerTestFactory.VORNAME).build()));

			assertThat(antragsteller).isNotEmpty();
		}

		@Test
		void shouldNotBeEmptyIfHasNachname() {
			var antragsteller = callCreator(buildVorgang(Antragsteller.builder().nachname(AntragstellerTestFactory.NACHNAME).build()));

			assertThat(antragsteller).isNotEmpty();
		}

		@Test
		void shouldNotBeEmptyIfHasAnrede() {
			var antragsteller = callCreator(buildVorgang(Antragsteller.builder().anrede(AntragstellerTestFactory.ANREDE).build()));

			assertThat(antragsteller).isNotEmpty();
		}

		@Test
		void shouldNotBeEmptyIfHasGeburtsdatum() {
			try (var dateConverter = mockStatic(DateConverter.class)) {
				dateConverter.when(() -> DateConverter.convertGermanFormatToISO(AntragstellerTestFactory.GEBURTSDATUM_STR))
						.thenReturn(Optional.of(geburtsdatum));

				var antragsteller = callCreator(
						buildVorgang(Antragsteller.builder().geburtsdatum(AntragstellerTestFactory.GEBURTSDATUM_STR).build()));

				assertThat(antragsteller).isNotEmpty();
			}
		}

		private Optional<Antragsteller> callCreator(VorgangWithEingang vorgang) {
			return creator.getAntragstellerIfHasRequiredData(vorgang);
		}

		private VorgangWithEingang buildVorgang(Antragsteller antragsteller) {
			return VorgangWithEingangTestFactory.createBuilder().eingang(EingangTestFactory.createBuilder().antragsteller(antragsteller).build())
					.build();
		}
	}

}